@@ -1205,6 +1205,7 @@ gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=lis
   ## which will break the standard handling of the control list. Following line fixes.
   ## but actually Martin M has now fixed lmeControl...
   ##if (!is.null(control$sigma)&&control$sigma==0) control$sigma <- NULL
+  if (inherits(family,"extended.family")) warning("gamm is not designed to use extended families")
   control <- do.call("lmeControl",control) 
     # check that random is a named list
     if (!is.null(random))
@@ -1582,7 +1583,8 @@ gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=lis
     ## set environments to global to avoid enormous saved object files
     environment(attr(object$model,"terms")) <- 
     environment(object$terms) <- environment(object$pterms) <- 
-    environment(object$formula) <-environment(object$pred.formula) <-  .GlobalEnv
+    environment(object$formula) <- .GlobalEnv
+    if (!is.null(object$pred.formula)) environment(object$pred.formula) <-  .GlobalEnv
     ret$gam <- object
     environment(attr(ret$lme$data,"terms")) <- environment(ret$lme$terms) <- .GlobalEnv
     if (!is.null(ret$lme$modelStruct$varStruct)) {
