@@ -1,5 +1,5 @@
 #  File src/library/base/R/duplicated.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2014 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 duplicated <- function(x, incomparables = FALSE, ...) UseMethod("duplicated")
 
@@ -26,25 +26,26 @@ duplicated.default <-
 duplicated.data.frame <-
     function(x, incomparables = FALSE, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
     if(length(x) != 1L)
-        duplicated(do.call("paste", c(x, sep="\r")), fromLast = fromLast)
+        duplicated(do.call(Map, c(list, x)), fromLast = fromLast)
     else duplicated(x[[1L]], fromLast = fromLast, ...)
 }
 
 duplicated.matrix <- duplicated.array <-
     function(x, incomparables = FALSE, MARGIN = 1L, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
     dx <- dim(x)
     ndim <- length(dx)
     if (length(MARGIN) > ndim || any(MARGIN > ndim))
         stop(gettextf("MARGIN = %d is invalid for dim = %d", MARGIN, dx),
              domain = NA)
-    collapse <- (ndim > 1L) && (prod(dx[-MARGIN]) > 1L)
-    temp <- if(collapse) apply(x, MARGIN, function(x) paste(x, collapse = "\r")) else x
+    temp <- if((ndim > 1L) && (prod(dx[-MARGIN]) > 1L))
+                apply(x, MARGIN, list)
+            else x
     res <- duplicated.default(temp, fromLast = fromLast, ...)
     dim(res) <- dim(temp)
     dimnames(res) <- dimnames(temp)
@@ -62,23 +63,24 @@ anyDuplicated.default <-
 anyDuplicated.data.frame <-
     function(x, incomparables = FALSE, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
-    anyDuplicated(do.call("paste", c(x, sep="\r")), fromLast = fromLast)
+    anyDuplicated(do.call(Map, c(list, x)), fromLast = fromLast)
 }
 
 anyDuplicated.matrix <- anyDuplicated.array <-
     function(x, incomparables = FALSE, MARGIN = 1L, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
     dx <- dim(x)
     ndim <- length(dx)
     if (length(MARGIN) > ndim || any(MARGIN > ndim))
         stop(gettextf("MARGIN = %d is invalid for dim = %d", MARGIN, dx),
              domain = NA)
-    collapse <- (ndim > 1L) && (prod(dx[-MARGIN]) > 1L)
-    temp <- if(collapse) apply(x, MARGIN, function(x) paste(x, collapse = "\r")) else x
+    temp <- if((ndim > 1L) && (prod(dx[-MARGIN]) > 1L))
+                apply(x, MARGIN, list)
+            else x
     anyDuplicated.default(temp, fromLast = fromLast)
 }
 
@@ -106,7 +108,7 @@ unique.default <-
 
 unique.data.frame <- function(x, incomparables = FALSE, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
     x[!duplicated(x, fromLast = fromLast, ...),  , drop = FALSE]
 }
@@ -114,15 +116,16 @@ unique.data.frame <- function(x, incomparables = FALSE, fromLast = FALSE, ...)
 unique.matrix <- unique.array <-
     function(x, incomparables = FALSE , MARGIN = 1, fromLast = FALSE, ...)
 {
-    if(!identical(incomparables, FALSE))
+    if(!isFALSE(incomparables))
 	.NotYetUsed("incomparables != FALSE")
     dx <- dim(x)
     ndim <- length(dx)
     if (length(MARGIN) > ndim || any(MARGIN > ndim))
         stop(gettextf("MARGIN = %d is invalid for dim = %d", MARGIN, dx),
              domain = NA)
-    collapse <- (ndim > 1L) && (prod(dx[-MARGIN]) > 1L)
-    temp <- if(collapse) apply(x, MARGIN, function(x) paste(x, collapse = "\r")) else x
+    temp <- if((ndim > 1L) && (prod(dx[-MARGIN]) > 1L))
+                apply(x, MARGIN, list)
+            else x
     args <- rep(alist(a=), ndim)
     names(args) <- NULL
     args[[MARGIN]] <- !duplicated.default(temp, fromLast = fromLast, ...)
