@@ -1,7 +1,7 @@
 #  File src/library/base/R/print.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 print <- function(x, ...) UseMethod("print")
 
@@ -40,9 +40,11 @@ prmatrix <-
     .Internal(prmatrix(x, rowlab, collab, quote, right, na.print))
 }
 
-noquote <- function(obj) {
+noquote <- function(obj, right = FALSE) {
     ## constructor for a useful "minor" class
-    if(!inherits(obj,"noquote")) class(obj) <- c(attr(obj, "class"),"noquote")
+    if(!inherits(obj,"noquote"))
+        class(obj) <- c(attr(obj, "class"),
+                        if(right) c(right = "noquote") else "noquote")
     obj
 }
 
@@ -63,12 +65,23 @@ c.noquote <- function(..., recursive = FALSE)
 }
 
 print.noquote <- function(x, ...) {
-    if(!is.null(cl <- attr(x, "class"))) {
-	cl <- cl[cl != "noquote"]
-        attr(x, "class") <-
-          (if(length(cl)) cl else NULL)
-      }
-    print(x, quote = FALSE, ...)
+    rgt.in.dots <- any("right" == names(list(...)))
+    if(copy <- !is.null(cl <- attr(x, "class"))) {
+	isNQ <- cl == "noquote"
+	if(!rgt.in.dots)
+	    right <- any("right" == names(cl[isNQ]))
+	if(copy <- any(isNQ)) {
+	    ox <- x
+	    cl <- cl[!isNQ]
+	    attr(x, "class") <- if(length(cl)) cl # else NULL
+	}
+    } else
+	right <- FALSE
+    if(rgt.in.dots)
+	print(x, quote = FALSE, ...)
+    else
+	print(x, quote = FALSE, right = right, ...)
+    invisible(if(copy) ox else x)
 }
 
 ## for alias.lm, aov
