@@ -1,5 +1,7 @@
 #  File src/library/base/R/structure.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
+#
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -12,24 +14,27 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## This remaps special names as they are used by deparsing, but why are they?
 ##
 structure <- function (.Data, ...)
 {
+    if(is.null(.Data))
+        warning("Calling 'structure(NULL, *)' is deprecated, as NULL cannot have attributes.\n  Consider 'structure(list(), *)' instead.")
+        ## to become: stop("attempt to set an attribute on NULL")
     attrib <- list(...)
     if(length(attrib)) {
         specials <- c(".Dim", ".Dimnames", ".Names", ".Tsp", ".Label")
         replace <- c("dim", "dimnames", "names", "tsp", "levels")
 	m <- match(names(attrib), specials)
-	ok <- (!is.na(m) & m)
+	ok <- !is.na(m)
 	names(attrib)[ok] <- replace[m[ok]]
         ## prior to 2.5.0 factors would deparse to double codes
 	if("factor" %in% attrib[["class", exact = TRUE]]
            && typeof(.Data) == "double")
-	   storage.mode(.Data) <- "integer"
+            storage.mode(.Data) <- "integer"
 	attributes(.Data) <- c(attributes(.Data), attrib)
     }
-    return(.Data)
+    .Data
 }
