@@ -1,7 +1,7 @@
 #  File src/library/stats/R/contr.poly.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 contr.poly <- function (n, scores = 1:n, contrasts = TRUE, sparse = FALSE)
 {
@@ -67,23 +67,24 @@ poly <- function(x, ..., degree = 1, coefs = NULL, raw = FALSE, simple = FALSE)
 {
     dots <- list(...)
     if(nd <- length(dots)) {
-        if(nd == 1 && length(dots[[1L]]) == 1L) # unnamed degree
+	dots_deg <- nd == 1L && length(dots[[1L]]) == 1L
+	if(dots_deg) # unnamed degree, nothing else in '...'
             degree <- dots[[1L]]
         else return(polym(x, ..., degree = degree, coefs=coefs, raw = raw))
     }
-    if(is.matrix(x)) { ## FIXME: fails when combined with 'unnamed degree' above
-        m <- unclass(as.data.frame(cbind(x, ...)))
+    if(is.matrix(x)) {
+	m <- unclass(as.data.frame(if(nd && dots_deg) x else cbind(x, ...)))
 	return(do.call(polym, c(m, degree = degree, raw = raw,
 				list(coefs=coefs))))
     }
     if(degree < 1)
         stop("'degree' must be at least 1")
-    if(anyNA(x)) stop("missing values are not allowed in 'poly'")
     if(raw) {
         Z <- outer(x, 1L:degree, "^")
         colnames(Z) <- 1L:degree
     } else {
 	if(is.null(coefs)) { # fitting
+	    if(anyNA(x)) stop("missing values are not allowed in 'poly'")
 	    if(degree >= length(unique(x)))
 		stop("'degree' must be less than number of unique points")
 	    xbar <- mean(x)
@@ -113,11 +114,7 @@ poly <- function(x, ..., degree = 1, coefs = NULL, raw = FALSE, simple = FALSE)
         if(!simple) ## we may want to use the prediction to clone another prediction
             attr(Z, "coefs") <- list(alpha = alpha, norm2 = norm2)
     }
-    if(!simple) {
-        attr(Z, "degree") <- 1L:degree
-        class(Z) <- c("poly", "matrix")
-    }
-    Z
+    if(simple) Z else structure(Z, degree = 1L:degree, class = c("poly", "matrix"))
 }
 
 predict.poly <- function(object, newdata, ...)
