@@ -1,7 +1,7 @@
 #  File src/library/stats/R/lm.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -323,11 +323,9 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
     ans <- z[c("call", "terms", if(!is.null(z$weights)) "weights")]
     ans$residuals <- r
     ans$coefficients <-
-	cbind(est, se, tval, 2*pt(abs(tval), rdf, lower.tail = FALSE))
-    dimnames(ans$coefficients) <-
-	list(names(z$coefficients)[Qr$pivot[p1]],
-	     c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
-    ans$aliased <- is.na(coef(object))  # used in print method
+	cbind(Estimate = est, "Std. Error" = se, "t value" = tval,
+	      "Pr(>|t|)" = 2*pt(abs(tval), rdf, lower.tail = FALSE))
+    ans$aliased <- is.na(z$coefficients)  # used in print method
     ans$sigma <- sqrt(resvar)
     ans$df <- c(p, rdf, NCOL(Qr$qr))
     if (p != attr(z$terms, "intercept")) {
@@ -386,7 +384,7 @@ print.summary.lm <-
                 " not defined because of singularities)\n", sep = "")
         else cat("\nCoefficients:\n")
         coefs <- x$coefficients
-        if(!is.null(aliased <- x$aliased) && any(aliased)) {
+        if(any(aliased <- x$aliased)) {
             cn <- names(aliased)
             coefs <- matrix(NA, length(aliased), 4, dimnames=list(cn, colnames(coefs)))
             coefs[!aliased, ] <- x$coefficients
@@ -469,16 +467,24 @@ simulate.lm <- function(object, nsim = 1, seed = NULL, ...)
         RNGstate <- structure(seed, kind = as.list(RNGkind()))
         on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
     }
+    fam <- if(inherits(object, "glm")) object$family$family else "gaussian"
     ftd <- fitted(object)             # == napredict(*, object$fitted)
-    nm <- names(ftd)
+    isMlm <- identical(fam, "gaussian") && is.matrix(ftd)
+    nm <- if(isMlm) dimnames(ftd) else names(ftd)
+    if(isMlm) ## Not hard. Biggest question: how exactly the data frame should look
+	stop("simulate() is not yet implemented for multivariate lm()")
     n <- length(ftd)
     ntot <- n * nsim
-    fam <- if(inherits(object, "glm")) object$family$family else "gaussian"
     val <- switch(fam,
                   "gaussian" = {
                       vars <- deviance(object)/ df.residual(object)
-                      if (!is.null(object$weights)) vars <- vars/object$weights
-                      ftd + rnorm(ntot, sd = sqrt(vars))
+                      if(isMlm) {
+                          ## _TODO_
+                          ## weights ==> "vars / weights" as matrix with  dim(ftd)
+                      } else {
+                          if (!is.null(object$weights)) vars <- vars/object$weights
+                          ftd + rnorm(ntot, sd = sqrt(vars))
+                      }
                   },
                   if(!is.null(object$family$simulate))
                       object$family$simulate(object, nsim)
@@ -486,12 +492,15 @@ simulate.lm <- function(object, nsim = 1, seed = NULL, ...)
                             domain = NA)
                   )
 
-    if(!is.list(val)) {
+    if(isMlm) {
+        ## _TODO_
+    } else if(!is.list(val)) {
         dim(val) <- c(n, nsim)
         val <- as.data.frame(val)
     } else
         class(val) <- "data.frame"
-    names(val) <- paste("sim", seq_len(nsim), sep="_")
+    ## isMlm: conceptually, each "sim_i" could be a *matrix* [unusually]
+    names(val) <- paste0("sim_", seq_len(nsim))
     if (!is.null(nm)) row.names(val) <- nm
     attr(val, "seed") <- RNGstate
     val
@@ -628,8 +637,8 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
                             c("Res.Df", "RSS", "Df", "Sum of Sq"))
 
     title <- "Analysis of Variance Table\n"
-    topnote <- paste("Model ", format(1L:nmodels),": ",
-		     variables, sep = "", collapse = "\n")
+    topnote <- paste0("Model ", format(1L:nmodels), ": ", variables,
+                      collapse = "\n")
 
     ## calculate test statistic if needed
 
