@@ -1,7 +1,7 @@
 #  File src/library/stats/R/optim.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2000-12 The R Core Team
+#  Copyright (C) 2000-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 optim <-
     function(par, fn, gr = NULL, ...,
@@ -37,7 +37,7 @@ optim <-
 		ndeps = rep.int(1e-3, npar),
 		maxit = 100L, abstol = -Inf, reltol = sqrt(.Machine$double.eps),
 		alpha = 1.0, beta = 0.5, gamma = 2.0,
-		REPORT = 10,
+		REPORT = 10, warn.1d.NelderMead = TRUE,
 		type = 1,
 		lmm = 5, factr = 1e7, pgtol = 0,
 		tmax = 10, temp = 10.0)
@@ -57,7 +57,7 @@ optim <-
     if (method == "L-BFGS-B" &&
 	any(!is.na(match(c("reltol","abstol"), namc))))
 	warning("method L-BFGS-B uses 'factr' (and 'pgtol') instead of 'reltol' and 'abstol'")
-    if(npar == 1 && method == "Nelder-Mead")
+    if(npar == 1 && method == "Nelder-Mead" && isTRUE(con$warn.1d.NelderMead))
         warning("one-dimensional optimization by Nelder-Mead is unreliable:\nuse \"Brent\" or optimize() directly")
     if(npar > 1 && method == "Brent")
 	stop('method = "Brent" is only available for one-dimensional optimization')
