@@ -1,5 +1,5 @@
 #  File src/library/tools/R/pdftools.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2014 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## See PDF Reference version 1.7 chapter 3:
 ##   At the most fundamental level, a PDF file is a sequence of 8-bit
@@ -237,7 +237,7 @@ function(file, cache = TRUE)
             pos <- .con_seek(con)
             pdf_read_object_header(con)
             obj <- pdf_read_object(con)
-            if(!("Type" %in% names(obj)) ||
+            if(("Type" %notin% names(obj)) ||
                !(obj[["Type"]] == "XRef")) {
                 ## Something's wrong.
                 ## Try finding the xref table before pos.
