@@ -52,7 +52,7 @@ SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP ind = PROTECT(allocVector(realIndx ? REALSXP : INTSXP, 1));
     SEXP isym = install("i");
     defineVar(isym, ind, rho);
-    SET_NAMED(ind, 1);
+    INCREMENT_NAMED(ind);
 
     /* Notice that it is OK to have one arg to LCONS do memory
        allocation and not PROTECT the result (LCONS does memory
@@ -138,7 +138,7 @@ SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 	SEXP isym = install("i");
 	PROTECT(ind = allocVector(realIndx ? REALSXP : INTSXP, 1));
 	defineVar(isym, ind, rho);
-	SET_NAMED(ind, 1);
+	INCREMENT_NAMED(ind);
 
 	/* Notice that it is OK to have one arg to LCONS do memory
 	   allocation and not PROTECT the result (LCONS does memory
@@ -352,11 +352,11 @@ SEXP attribute_hidden do_rapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 static Rboolean islistfactor(SEXP X)
 {
     int i, n = length(X);
-
-    if(n == 0) return FALSE;
+    
     switch(TYPEOF(X)) {
     case VECSXP:
     case EXPRSXP:
+        if(n == 0) return NA_LOGICAL;
 	for(i = 0; i < LENGTH(X); i++)
 	    if(!islistfactor(VECTOR_ELT(X, i))) return FALSE;
 	return TRUE;
@@ -383,11 +383,11 @@ SEXP attribute_hidden do_islistfactor(SEXP call, SEXP op, SEXP args, SEXP rho)
 	goto do_ans;
     }
     if(!recursive) {
-    for(i = 0; i < LENGTH(X); i++)
-	if(!isFactor(VECTOR_ELT(X, i))) {
-	    lans = FALSE;
-	    break;
-	}
+	for(i = 0; i < LENGTH(X); i++)
+	    if(!isFactor(VECTOR_ELT(X, i))) {
+		lans = FALSE;
+		break;
+	    }
     } else {
 	switch(TYPEOF(X)) {
 	case VECSXP:
@@ -396,11 +396,15 @@ SEXP attribute_hidden do_islistfactor(SEXP call, SEXP op, SEXP args, SEXP rho)
 	default:
 	    goto do_ans;
 	}
-	for(i = 0; i < LENGTH(X); i++)
-	    if(!islistfactor(VECTOR_ELT(X, i))) {
+        lans = FALSE;
+	for(i = 0; i < LENGTH(X); i++) {
+            Rboolean isfactor = islistfactor(VECTOR_ELT(X, i));
+	    if(!isfactor) {
 		lans = FALSE;
 		break;
-	    }
+	    } else if (isfactor == TRUE)
+                lans = TRUE;
+        }
     }
 do_ans:
     return ScalarLogical(lans);
