@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2002-3     The R Foundation
- *  Copyright (C) 1999-2015  The R Core Team.
+ *  Copyright (C) 2002-2017  The R Foundation
+ *  Copyright (C) 1999-2018  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -165,6 +165,54 @@ void R_warn_S3_for_S4(SEXP method) {
 }
 #endif
 
+static SEXP findFunInEnvRange(SEXP symbol, SEXP rho, SEXP target)
+{
+    SEXP vl;
+    while(rho != R_EmptyEnv) {
+	vl = findVarInFrame3(rho, symbol, TRUE);
+	if (vl != R_UnboundValue) {
+	    if (TYPEOF(vl) == PROMSXP) {
+		PROTECT(vl);
+		vl = eval(vl, rho);
+		UNPROTECT(1);
+	    }
+	    if ((TYPEOF(vl) == CLOSXP ||
+		 TYPEOF(vl) == BUILTINSXP ||
+		 TYPEOF(vl) == SPECIALSXP))
+		return (vl);
+	}
+	if(rho == target)
+	    return (R_UnboundValue);
+	else
+	    rho = ENCLOS(rho);
+    }
+    return (R_UnboundValue);
+}
+
+static SEXP findFunWithBaseEnvAfterGlobalEnv(SEXP symbol, SEXP rho)
+{
+    SEXP vl;
+    while(rho != R_EmptyEnv) {
+	vl = findVarInFrame3(rho, symbol, TRUE);
+	if (vl != R_UnboundValue) {
+	    if (TYPEOF(vl) == PROMSXP) {
+		PROTECT(vl);
+		vl = eval(vl, rho);
+		UNPROTECT(1);
+	    }
+	    if ((TYPEOF(vl) == CLOSXP ||
+		 TYPEOF(vl) == BUILTINSXP ||
+		 TYPEOF(vl) == SPECIALSXP))
+		return (vl);
+	}
+	if(rho == R_GlobalEnv)
+	    rho = R_BaseEnv;
+	else
+	    rho = ENCLOS(rho);
+    }
+    return (R_UnboundValue);
+}
+
 /*  usemethod  -  calling functions need to evaluate the object
  *  (== 2nd argument).	They also need to ensure that the
  *  argument list is set up in the correct manner.
@@ -183,8 +231,10 @@ void R_warn_S3_for_S4(SEXP method) {
 attribute_hidden
 SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 {
-    SEXP val;
+    SEXP val, top = R_NilValue;	/* -Wall */
     static SEXP s_S3MethodsTable = NULL;
+    static int lookup_baseenv_after_globalenv = -1;
+    char *lookup;
 
     if (TYPEOF(callrho) != ENVSXP) {
 	if (TYPEOF(callrho) == NILSXP)
@@ -201,33 +251,56 @@ SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 	    error(_("bad generic definition environment"));
     }
 
+    if(lookup_baseenv_after_globalenv == -1) {
+	lookup = getenv("_R_S3_METHOD_LOOKUP_BASEENV_AFTER_GLOBALENV_");
+	lookup_baseenv_after_globalenv = 
+	    ((lookup != NULL) && StringTrue(lookup)) ? 1 : 0;
+    }
+
     /* This evaluates promises */
-    val = findVar1(method, callrho, FUNSXP, TRUE);
-    if (isFunction(val))
+    PROTECT(top = topenv(R_NilValue, callrho));
+    val = findFunInEnvRange(method, callrho, top);
+    if(val != R_UnboundValue) {
+	UNPROTECT(1);
 	return val;
-    else {
-	/* We assume here that no one registered a non-function */
-	if (!s_S3MethodsTable)
-	    s_S3MethodsTable = install(".__S3MethodsTable__.");
-	SEXP table = findVarInFrame3(defrho,
-				     s_S3MethodsTable,
-				     TRUE);
-	if (TYPEOF(table) == PROMSXP) {
-	    PROTECT(table);
-	    table = eval(table, R_BaseEnv);
+    }
+
+    /* We assume here that no one registered a non-function */
+    if (!s_S3MethodsTable)
+	s_S3MethodsTable = install(".__S3MethodsTable__.");
+    SEXP table = findVarInFrame3(defrho, s_S3MethodsTable, TRUE);
+    if (TYPEOF(table) == PROMSXP) {
+	PROTECT(table);
+	table = eval(table, R_BaseEnv);
+	UNPROTECT(1);
+    }
+    if (TYPEOF(table) == ENVSXP) {
+	PROTECT(table);
+	val = findVarInFrame3(table, method, TRUE);
+	UNPROTECT(1);
+	if (TYPEOF(val) == PROMSXP) {
+	    PROTECT(val);
+	    val = eval(val, rho);
 	    UNPROTECT(1);
 	}
-	if (TYPEOF(table) == ENVSXP) {
-	    val = findVarInFrame3(table, method, TRUE);
-	    if (TYPEOF(val) == PROMSXP) {
-		PROTECT(val);
-		val = eval(val, rho);
-		UNPROTECT(1);
-	    }
+	if(val != R_UnboundValue) {
+	    UNPROTECT(1);
 	    return val;
 	}
-	return R_UnboundValue;
+    } 
+
+    if(lookup_baseenv_after_globalenv) {
+	if (top == R_GlobalEnv)
+	    top = R_BaseEnv;
+	else
+	    top = ENCLOS(top);
+	val = findFunWithBaseEnvAfterGlobalEnv(method, top);
     }
+    else
+	val = findFunInEnvRange(method, ENCLOS(top), R_EmptyEnv);
+    UNPROTECT(1);
+
+    return val;
 }
 
 #ifdef UNUSED
@@ -535,7 +608,7 @@ static R_INLINE SEXP getPrimitive(SEXP symbol)
 	PROTECT(value);
 	value = eval(value, R_GlobalEnv);
 	UNPROTECT(1);
-	SET_NAMED(value, 2);
+	ENSURE_NAMEDMAX(value);
     }
     if (TYPEOF(value) == BUILTINSXP || TYPEOF(value) == SPECIALSXP)
         return value;
@@ -555,7 +628,6 @@ static R_INLINE SEXP getPrimitive(SEXP symbol)
 /* the second the object and any remaining are matched with the */
 /* formals of the chosen method. */
 
-#define ARGUSED(x) LEVELS(x)
 
 /* This is a special .Internal */
 SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
@@ -583,8 +655,8 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 
     PROTECT(newcall = duplicate(cptr->call));
 
-    /* eg get("print.ts")(1) */
-    if (TYPEOF(CAR(cptr->call)) == LANGSXP)
+    /* eg get("print.ts")(1) or do.call() */
+    if (TYPEOF(CAR(cptr->call)) != SYMSXP)
        error(_("'NextMethod' called from an anonymous function"));
 
     readS3VarsFromFrame(sysp, &generic, &group, &klass, &method,
@@ -829,7 +901,32 @@ SEXP attribute_hidden do_unclass(SEXP call, SEXP op, SEXP args, SEXP env)
     return CAR(args);
 }
 
+/** Version of inherits() that supports S4 inheritance and implicit
+    classes.  The inlined inherits() does not have access to private
+    entry points R_data_class() and R_data_class2(). The semantics of
+    inherits2() are identical to that of the R-level inherits(),
+    except there is no translation.
+*/
 
+Rboolean attribute_hidden inherits2(SEXP x, const char *what) {
+    if (OBJECT(x)) {
+	SEXP klass;
+
+	if(IS_S4_OBJECT(x))
+	    PROTECT(klass = R_data_class2(x));
+	else
+	    PROTECT(klass = R_data_class(x, FALSE));
+	int nclass = length(klass);
+	for (int i = 0; i < nclass; i++) {
+	    if (!strcmp(CHAR(STRING_ELT(klass, i)), what)) {
+		UNPROTECT(1);
+		return TRUE;
+	    }
+	}
+	UNPROTECT(1);
+    }
+    return FALSE;
+}
 
 /* NOTE: Fast  inherits(x, what)    in ../include/Rinlinedfuns.h
  * ----        ----------------- */
@@ -858,13 +955,7 @@ static SEXP inherits3(SEXP x, SEXP what, SEXP which)
 
     if( !isLogical(which) || (LENGTH(which) != 1) )
 	error(_("'which' must be a length 1 logical vector"));
-    int isvec = asLogical(which);
-
-#ifdef _be_too_picky_
-    if(IS_S4_OBJECT(x) && nwhat == 1 && !isvec &&
-       !isNull(R_getClassDef(translateChar(STRING_ELT(what, 0)))))
-	warning(_("use 'is()' instead of 'inherits()' on S4 objects"));
-#endif
+    Rboolean isvec = asLogical(which);
 
     if(isvec)
 	PROTECT(rval = allocVector(INTSXP, nwhat));
@@ -936,19 +1027,21 @@ int R_check_class_and_super(SEXP x, const char **valid, SEXP rho)
 	   .selectSuperClasses(getClass("....")@contains, dropVirtual=TRUE)  */
 	SEXP classExts, superCl, _call;
 	static SEXP s_contains = NULL, s_selectSuperCl = NULL;
-	int i;
 	if(!s_contains) {
 	    s_contains      = install("contains");
 	    s_selectSuperCl = install(".selectSuperClasses");
 	}
 	SEXP classDef = PROTECT(R_getClassDef(class));
 	PROTECT(classExts = R_do_slot(classDef, s_contains));
-	PROTECT(_call = lang3(s_selectSuperCl, classExts,
-			      /* dropVirtual = */ ScalarLogical(1)));
+	/* .selectSuperClasses(getClassDef(class)@contains, dropVirtual = TRUE,
+	 *                     namesOnly = TRUE, directOnly = FALSE, simpleOnly = TRUE) :
+	 */
+	PROTECT(_call = lang6(s_selectSuperCl, classExts, ScalarLogical(1),
+			      ScalarLogical(1), ScalarLogical(0), ScalarLogical(1)));
 	superCl = eval(_call, rho);
 	UNPROTECT(3); /* _call, classExts, classDef */
 	PROTECT(superCl);
-	for(i=0; i < LENGTH(superCl); i++) {
+	for(int i=0; i < LENGTH(superCl); i++) {
 	    const char *s_class = CHAR(STRING_ELT(superCl, i));
 	    for (ans = 0; ; ans++) {
 		if (!strlen(valid[ans]))
@@ -1189,7 +1282,7 @@ SEXP R_set_prim_method(SEXP fname, SEXP op, SEXP code_vec, SEXP fundef,
     }
     if (!isPrimitive(op)) {
         SEXP internal = R_do_slot(op, install("internal"));
-        op = INTERNAL(install(CHAR(asChar(internal))));
+        op = INTERNAL(installTrChar(asChar(internal)));
         if (op == R_NilValue) {
           error("'internal' slot does not name an internal function: %s",
                 CHAR(asChar(internal)));
@@ -1431,7 +1524,7 @@ SEXP attribute_hidden
 R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
 		    Rboolean promisedArgs)
 {
-    SEXP fundef, value, mlist=R_NilValue, s, a, b;
+    SEXP fundef, value, mlist=R_NilValue, s, a, b, suppliedvars;
     int offset;
     prim_methods_t current;
     offset = PRIMOFFSET(op);
@@ -1462,17 +1555,25 @@ R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
             if (inherits(value, "internalDispatchMethod")) {
                 return(NULL);
             }
+            PROTECT(suppliedvars = list1(mkString(PRIMNAME(op))));
+            SET_TAG(suppliedvars, R_dot_Generic);
 	    /* found a method, call it with promised args */
 	    if(!promisedArgs) {
 		PROTECT(s = promiseArgs(CDR(call), rho));
 		if (length(s) != length(args)) error(_("dispatch error"));
 		for (a = args, b = s; a != R_NilValue; a = CDR(a), b = CDR(b))
 		    SET_PRVALUE(CAR(b), CAR(a));
-		value =  applyClosure(call, value, s, rho, R_NilValue);
-		UNPROTECT(1);
+		value =  applyClosure(call, value, s, rho, suppliedvars);
+#ifdef ADJUST_ENVIR_REFCNTS
+		unpromiseArgs(s);
+#endif
+		UNPROTECT(2);
 		return value;
-	    } else
-		return applyClosure(call, value, args, rho, R_NilValue);
+	    } else {
+		value = applyClosure(call, value, args, rho, suppliedvars);
+                UNPROTECT(1);
+                return value;
+            }
 	}
 	/* else, need to perform full method search */
     }
@@ -1583,7 +1684,10 @@ SEXP R_do_new_object(SEXP class_def)
     }
     PROTECT(e = R_do_slot(class_def, s_className));
     PROTECT(value = duplicate(R_do_slot(class_def, s_prototype)));
-    if(TYPEOF(value) == S4SXP || getAttrib(e, R_PackageSymbol) != R_NilValue)
+    Rboolean xDataType = TYPEOF(value) == ENVSXP || TYPEOF(value) == SYMSXP ||
+	TYPEOF(value) == EXTPTRSXP;
+    if((TYPEOF(value) == S4SXP || getAttrib(e, R_PackageSymbol) != R_NilValue) &&
+       !xDataType)
     { /* Anything but an object from a base "class" (numeric, matrix,..) */
 	setAttrib(value, R_ClassSymbol, e);
 	SET_S4_OBJECT(value);
