@@ -195,13 +195,7 @@ static void PrintLanguageEtc(SEXP s, Rboolean useSource, Rboolean isClosure)
     }
     PROTECT(t);
     for (i = 0; i < LENGTH(t); i++) {
-#ifdef rev72839__not_ok_eg_for_escaped_chars
-	// did solve PR#16732
-	const char *ctmp = EncodeString(STRING_ELT(t, i),  0, 0, Rprt_adj_none);
-	Rprintf("%s\n", ctmp); /* translated */
-#else
- 	Rprintf("%s\n", CHAR(STRING_ELT(t, i))); /* translated */
-#endif
+ 	Rprintf("%s\n", translateChar(STRING_ELT(t, i))); // translate: for srcref part (PR#16732)
     }
     UNPROTECT(1);
     if (isClosure) {
@@ -344,9 +338,10 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		break;
 	    case LGLSXP:
 		if (LENGTH(tmp) == 1) {
-		    formatLogical(LOGICAL(tmp), 1, &w);
+		    const int *x = LOGICAL_RO(tmp);
+		    formatLogical(x, 1, &w);
 		    snprintf(pbuf, 115, "%s",
-			     EncodeLogical(LOGICAL(tmp)[0], w));
+			     EncodeLogical(x[0], w));
 		} else
 		    snprintf(pbuf, 115, "Logical,%d", LENGTH(tmp));
 		break;
@@ -356,24 +351,26 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		    snprintf(pbuf, 115, "factor,%d", LENGTH(tmp));
 		} else {
 		    if (LENGTH(tmp) == 1) {
-			formatInteger(INTEGER(tmp), 1, &w);
+			const int *x = INTEGER_RO(tmp);
+			formatInteger(x, 1, &w);
 			snprintf(pbuf, 115, "%s",
-				 EncodeInteger(INTEGER(tmp)[0], w));
+				 EncodeInteger(x[0], w));
 		    } else
 			snprintf(pbuf, 115, "Integer,%d", LENGTH(tmp));
 		}
 		break;
 	    case REALSXP:
 		if (LENGTH(tmp) == 1) {
-		    formatReal(REAL(tmp), 1, &w, &d, &e, 0);
+		    const double *x = REAL_RO(tmp);
+		    formatReal(x, 1, &w, &d, &e, 0);
 		    snprintf(pbuf, 115, "%s",
-			     EncodeReal0(REAL(tmp)[0], w, d, e, OutDec));
+			     EncodeReal0(x[0], w, d, e, OutDec));
 		} else
 		    snprintf(pbuf, 115, "Numeric,%d", LENGTH(tmp));
 		break;
 	    case CPLXSXP:
 		if (LENGTH(tmp) == 1) {
-		    Rcomplex *x = COMPLEX(tmp);
+		    const Rcomplex *x = COMPLEX_RO(tmp);
 		    if (ISNA(x[0].r) || ISNA(x[0].i))
 			/* formatReal(NA) --> w=R_print.na_width, d=0, e=0 */
 			snprintf(pbuf, 115, "%s",
