@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1997-2017   The R Core Team
  *  Copyright (C) 1995-1996   Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2014   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <R_ext/Riconv.h>
 #include <Rinterface.h>
 #include <errno.h>
+#include <rlocale.h>
 
 /*
   See ../unix/system.txt for a description of some of these functions.
@@ -50,6 +51,8 @@
 # include <sys/stat.h>
 #endif
 
+static int isDir(char *path);
+
 #ifdef HAVE_AQUA
 int (*ptr_CocoaSystem)(const char*);
 #endif
@@ -104,11 +107,11 @@ Rboolean attribute_hidden R_HiddenFile(const char *name)
 
 static char * fixmode(const char *mode)
 {
-    /* Rconnection can have a mode of 4 chars plus a null; we might
-     * add one char */
-    static char fixedmode[6];
-    fixedmode[4] = '\0';
-    strncpy(fixedmode, mode, 4);
+    /* Rconnection can have a mode of 4 chars plus a ccs= setting plus a null; we might
+     * add one char if neither b nor t is specified. */
+    static char fixedmode[20];
+    fixedmode[19] = '\0';
+    strncpy(fixedmode, mode, 19);
     if (!strpbrk(fixedmode, "bt")) {
 	strcat(fixedmode, "t");
     }
@@ -117,9 +120,9 @@ static char * fixmode(const char *mode)
 
 static wchar_t * wcfixmode(const wchar_t *mode)
 {
-    static wchar_t wcfixedmode[6];
-    wcfixedmode[4] = L'\0';
-    wcsncpy(wcfixedmode, mode, 4);
+    static wchar_t wcfixedmode[20];
+    wcfixedmode[19] = L'\0';
+    wcsncpy(wcfixedmode, mode, 19);
     if (!wcspbrk(wcfixedmode, L"bt")) {
 	wcscat(wcfixedmode, L"t");
     }
@@ -161,7 +164,12 @@ wchar_t *filenameToWchar(const SEXP fn, const Rboolean expand)
 	wcscpy(filename, L"");
 	return filename;
     }
-    if(IS_LATIN1(fn)) from = "latin1";
+    if(IS_LATIN1(fn))
+#ifdef HAVE_ICONV_CP1252
+	from = "CP1252";
+#else
+	from = "latin1";
+#endif
     if(IS_UTF8(fn)) from = "UTF-8";
     if(IS_BYTES(fn)) error(_("encoding of a filename cannot be 'bytes'"));
     obj = Riconv_open("UCS-2LE", from);
@@ -229,6 +237,11 @@ SEXP attribute_hidden do_interactive(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_tempdir(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
+    Rboolean check = asLogical(CAR(args));
+    if(check && !isDir(R_TempDir)) {
+	R_TempDir = NULL;
+	R_reInitTempDir(/* die_on_fail = */ FALSE);
+    }
     return mkString(R_TempDir);
 }
 
@@ -366,11 +379,11 @@ SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
 	wchar_t **w;
 	for (i = 0, w = _wenviron; *w != NULL; i++, w++)
 	    n = max(n, wcslen(*w));
-	N = 3*n+1;
+	N = 4*n+1;
 	char buf[N];
 	PROTECT(ans = allocVector(STRSXP, i));
 	for (i = 0, w = _wenviron; *w != NULL; i++, w++) {
-	    wcstoutf8(buf, *w, N); buf[N-1] = '\0';
+	    wcstoutf8(buf, *w, sizeof(buf));
 	    SET_STRING_ELT(ans, i, mkCharCE(buf, CE_UTF8));
 	}
 #else
@@ -389,10 +402,10 @@ SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if (w == NULL)
 		SET_STRING_ELT(ans, j, STRING_ELT(CADR(args), 0));
 	    else {
-		int n = wcslen(w), N = 3*n+1; /* UCS-2 maps to <=3 UTF-8 */
+		int n = wcslen(w), N = 4*n+1; /* UTF-16 maps to <= 4 UTF-8 */
 		R_CheckStack2(N);
 		char buf[N];
-		wcstoutf8(buf, w, N); buf[N-1] = '\0'; /* safety */
+		wcstoutf8(buf, w, sizeof(buf));
 		SET_STRING_ELT(ans, j, mkCharCE(buf, CE_UTF8));
 	    }
 #else
@@ -639,8 +652,10 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 		SHALLOW_DUPLICATE_ATTRIB(ans, x);
 	    }
 	} else {
-	    if(TYPEOF(x) != STRSXP)
+	    if(TYPEOF(x) != STRSXP) {
+		Riconv_close(obj);
 		error(_("'x' must be a character vector"));
+	    }
 	    if(toRaw) {
 		PROTECT(ans = allocVector(VECSXP, LENGTH(x)));
 		SHALLOW_DUPLICATE_ATTRIB(ans, x);
@@ -654,8 +669,10 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 		if (TYPEOF(si) == NILSXP) {
 		    if (!toRaw) SET_STRING_ELT(ans, i, NA_STRING);
 		    continue;
-		} else if (TYPEOF(si) != RAWSXP)
+		} else if (TYPEOF(si) != RAWSXP) {
+		    Riconv_close(obj);
 		    error(_("'x' must be a character vector or a list of NULL or raw vectors"));
+		}
 	    } else {
 		si = STRING_ELT(x, i);
 		if (si == NA_STRING) {
@@ -791,7 +808,7 @@ int Riconv_close (void *cd)
 typedef enum {
     NT_NONE        = 0, /* no translation to native encoding is needed */
     NT_FROM_UTF8   = 1, /* need to translate from UTF8 */
-    NT_FROM_LATIN1 = 2, /* need to translated from latin1 */
+    NT_FROM_LATIN1 = 2, /* need to translate from latin1 */
 } nttype_t;
 
 /* Decides whether translation to native encoding is needed. */
@@ -823,21 +840,26 @@ static void translateToNative(const char *ans, R_StringBuffer *cbuff,
 	error(_("internal error: no translation needed"));
 
     void * obj;
-    const char *inbuf;
+    const char *inbuf, *from;
     char *outbuf;
     size_t inb, outb, res;
 
     if(ttype == NT_FROM_LATIN1) {
 	if(!latin1_obj) {
-	    obj = Riconv_open("", "latin1");
+#ifdef HAVE_ICONV_CP1252
+	    from = "CP1252";
+#else
+	    from = "latin1";
+#endif
+	    obj = Riconv_open("", from);
 	    /* should never happen */
 	    if(obj == (void *)(-1))
 #ifdef Win32
 		error(_("unsupported conversion from '%s' in codepage %d"),
-		      "latin1", localeCP);
+		      from, localeCP);
 #else
 		error(_("unsupported conversion from '%s' to '%s'"),
-		      "latin1", "");
+		      from, "");
 #endif
 	    latin1_obj = obj;
 	}
@@ -849,10 +871,10 @@ static void translateToNative(const char *ans, R_StringBuffer *cbuff,
 	    if(obj == (void *)(-1))
 #ifdef Win32
 		error(_("unsupported conversion from '%s' in codepage %d"),
-		      "latin1", localeCP);
+		      "UTF-8", localeCP);
 #else
 		error(_("unsupported conversion from '%s' to '%s'"),
-		      "latin1", "");
+		      "UTF-8", "");
 #endif
 	    utf8_obj = obj;
 	}
@@ -881,21 +903,23 @@ next_char:
 	    /* This must be the first byte */
 	    size_t clen;
 	    wchar_t wc;
+	    Rwchar_t ucs;
 	    clen = utf8toucs(&wc, inbuf);
 	    if(clen > 0 && inb >= clen) {
+	    	if (IS_HIGH_SURROGATE(wc))
+	    	    ucs = utf8toucs32(wc, inbuf);
+	    	else
+	    	    ucs = (Rwchar_t) wc;
 		inbuf += clen; inb -= clen;
-# ifndef Win32
-		if((unsigned int) wc < 65536) {
-# endif
+		if(ucs < 65536) {
 		// gcc 7 objects to this with unsigned int
-		    snprintf(outbuf, 9, "<U+%04X>", (unsigned short) wc);
+		    snprintf(outbuf, 9, "<U+%04X>", (unsigned short) ucs);
 		    outbuf += 8; outb -= 8;
-# ifndef Win32
 		} else {
-		    snprintf(outbuf, 13, "<U+%08X>", (unsigned int) wc);
+		    // Rwchar_t is usually unsigned int, but wchar_t need not be
+		    snprintf(outbuf, 13, "<U+%08X>", (unsigned int) ucs);
 		    outbuf += 12; outb -= 12;
 		}
-# endif
 	    } else {
 		snprintf(outbuf, 5, "<%02x>", (unsigned char)*inbuf);
 		outbuf += 4; outb -= 4;
@@ -917,7 +941,8 @@ next_char:
 const char *translateChar(SEXP x)
 {
     if(TYPEOF(x) != CHARSXP)
-	error(_("'%s' must be called on a CHARSXP"), "translateChar");
+	error(_("'%s' must be called on a CHARSXP, but got '%s'"),
+	      "translateChar", type2char(TYPEOF(x)));
     nttype_t t = needsTranslation(x);
     const char *ans = CHAR(x);
     if (t == NT_NONE) return ans;
@@ -935,9 +960,10 @@ const char *translateChar(SEXP x)
 SEXP installTrChar(SEXP x)
 {
     if(TYPEOF(x) != CHARSXP)
-	error(_("'%s' must be called on a CHARSXP"), "installTrChar");
+	error(_("'%s' must be called on a CHARSXP, but got '%s'"),
+	      "installTrChar", type2char(TYPEOF(x)));
     nttype_t t = needsTranslation(x);
-    if (t == NT_NONE) return installChar(x);
+    if (t == NT_NONE) return installNoTrChar(x);
 
     R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
     translateToNative(CHAR(x), &cbuff, t);
@@ -947,6 +973,12 @@ SEXP installTrChar(SEXP x)
     return Sans;
 }
 
+/* now translates, now same as installTrChar */
+SEXP installChar(SEXP x)
+{
+    return installTrChar(x);
+}
+
 /* This may return a R_alloc-ed result, so the caller has to manage the
    R_alloc stack */
 const char *translateChar0(SEXP x)
@@ -963,25 +995,33 @@ const char *translateCharUTF8(SEXP x)
 {
     void *obj;
     const char *inbuf, *ans = CHAR(x);
-    char *outbuf, *p;
+    char *outbuf, *p, *from = "";
     size_t inb, outb, res;
     R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
 
     if(TYPEOF(x) != CHARSXP)
-	error(_("'%s' must be called on a CHARSXP"), "translateCharUTF8");
+	error(_("'%s' must be called on a CHARSXP, but got '%s'"),
+	      "translateCharUTF8", type2char(TYPEOF(x)));
     if(x == NA_STRING) return ans;
     if(IS_UTF8(x)) return ans;
     if(IS_ASCII(x)) return ans;
     if(IS_BYTES(x))
 	error(_("translating strings with \"bytes\" encoding is not allowed"));
 
-    obj = Riconv_open("UTF-8", IS_LATIN1(x) ? "latin1" : "");
+    if (IS_LATIN1(x))
+#ifdef HAVE_ICONV_CP1252
+	from = "CP1252";
+#else
+	from = "latin1";
+#endif
+    obj = Riconv_open("UTF-8", from);
     if(obj == (void *)(-1))
 #ifdef Win32
 	error(_("unsupported conversion from '%s' in codepage %d"),
-	      "latin1", localeCP);
+	      from, localeCP);
 #else
-       error(_("unsupported conversion from '%s' to '%s'"), "latin1", "UTF-8");
+	error(_("unsupported conversion from '%s' to '%s'"),
+	      from, "UTF-8");
 #endif
     R_AllocStringBuffer(0, &cbuff);
 top_of_loop:
@@ -1037,7 +1077,7 @@ attribute_hidden /* but not hidden on Windows, where it was used in tcltk.c */
 const wchar_t *wtransChar(SEXP x)
 {
     void * obj;
-    const char *inbuf, *ans = CHAR(x);
+    const char *inbuf, *ans = CHAR(x), *from;
     char *outbuf;
     wchar_t *p;
     size_t inb, outb, res, top;
@@ -1052,10 +1092,15 @@ const wchar_t *wtransChar(SEXP x)
 
     if(IS_LATIN1(x)) {
 	if(!latin1_wobj) {
-	    obj = Riconv_open(TO_WCHAR, "latin1");
+#ifdef HAVE_ICONV_CP1252
+	    from = "CP1252";
+#else
+	    from = "latin1";
+#endif
+	    obj = Riconv_open(TO_WCHAR, from);
 	    if(obj == (void *)(-1))
 		error(_("unsupported conversion from '%s' to '%s'"),
-		      "latin1", TO_WCHAR);
+		      from, TO_WCHAR);
 	    latin1_wobj = obj;
 	} else
 	    obj = latin1_wobj;
@@ -1065,7 +1110,7 @@ const wchar_t *wtransChar(SEXP x)
 	    obj = Riconv_open(TO_WCHAR, "UTF-8");
 	    if(obj == (void *)(-1))
 		error(_("unsupported conversion from '%s' to '%s'"),
-		      "latin1", TO_WCHAR);
+		      "UTF-8", TO_WCHAR);
 	    utf8_wobj = obj;
 	} else
 	    obj = utf8_wobj;
@@ -1161,7 +1206,7 @@ const char *reEnc(const char *x, cetype_t ce_in, cetype_t ce_out, int subst)
     case CE_LATIN1: fromcode = "CP1252"; break;
 #else
     case CE_NATIVE: fromcode = ""; break;
-    case CE_LATIN1: fromcode = "latin1"; break;
+    case CE_LATIN1: fromcode = "latin1"; break; /* FIXME: allow CP1252? */
 #endif
     case CE_UTF8:   fromcode = "UTF-8"; break;
     default: return x;
@@ -1349,12 +1394,27 @@ next_char:
 void attribute_hidden
 invalidate_cached_recodings(void)
 {
-    latin1_obj = NULL;
-    utf8_obj = NULL;
-    ucsmb_obj = NULL;
+    if (latin1_obj) {
+	Riconv_close(latin1_obj);
+	latin1_obj = NULL;
+    }
+    if (utf8_obj) {
+	Riconv_close(utf8_obj);
+	utf8_obj = NULL;
+    }
+    if (ucsmb_obj) {
+	Riconv_close(ucsmb_obj);
+	ucsmb_obj = NULL;
+    }
 #ifdef Win32
-    latin1_wobj = NULL;
-    utf8_wobj=NULL;
+    if (latin1_wobj) {
+	Riconv_close(latin1_wobj);
+	latin1_wobj = NULL;
+    }
+    if (utf8_wobj) {
+	Riconv_close(utf8_wobj);
+	utf8_wobj = NULL;
+    }
 #endif
 }
 
@@ -1441,12 +1501,15 @@ mbtoucs(unsigned int *wc, const char *s, size_t n)
     if (status == (size_t) -1) {
 	switch(errno){
 	case EINVAL:
+	    Riconv_close(cd);
 	    return (size_t) -2;
 	case EILSEQ:
+	    Riconv_close(cd);
 	    return (size_t) -1;
 	case E2BIG:
 	    break;
 	default:
+	    Riconv_close(cd);
 	    errno = EILSEQ;
 	    return (size_t) -1;
 	}
@@ -1554,7 +1617,7 @@ extern char * mkdtemp (char *template);
 # include <ctype.h>
 #endif
 
-void attribute_hidden InitTempDir()
+void R_reInitTempDir(int die_on_fail)
 {
     char *tmp, *tm, tmp1[PATH_MAX+11], *p;
 #ifdef Win32
@@ -1562,6 +1625,12 @@ void attribute_hidden InitTempDir()
     int hasspace = 0;
 #endif
 
+#define ERROR_MAYBE_DIE(MSG_)			\
+    if(die_on_fail)				\
+	R_Suicide(MSG_);			\
+    else					\
+	errorcall(R_NilValue, MSG_)
+
     if(R_TempDir) return; /* someone else set it */
     tmp = NULL; /* getenv("R_SESSION_TMPDIR");   no longer set in R.sh */
     if (!tmp) {
@@ -1591,7 +1660,9 @@ void attribute_hidden InitTempDir()
 	snprintf(tmp1, PATH_MAX+11, "%s/RtmpXXXXXX", tm);
 #endif
 	tmp = mkdtemp(tmp1);
-	if(!tmp) R_Suicide(_("cannot create 'R_TempDir'"));
+	if(!tmp) {
+	    ERROR_MAYBE_DIE(_("cannot create 'R_TempDir'"));
+	}
 #ifndef Win32
 # ifdef HAVE_SETENV
 	if(setenv("R_SESSION_TMPDIR", tmp, 1))
@@ -1615,7 +1686,7 @@ void attribute_hidden InitTempDir()
     size_t len = strlen(tmp) + 1;
     p = (char *) malloc(len);
     if(!p)
-	R_Suicide(_("cannot allocate 'R_TempDir'"));
+	ERROR_MAYBE_DIE(_("cannot allocate 'R_TempDir'"));
     else {
 	R_TempDir = p;
 	strcpy(R_TempDir, tmp);
@@ -1623,6 +1694,10 @@ void attribute_hidden InitTempDir()
     }
 }
 
+void attribute_hidden InitTempDir() {
+    R_reInitTempDir(/* die_on_fail = */ TRUE);
+}
+
 char * R_tmpnam(const char * prefix, const char * tempdir)
 {
     return R_tmpnam2(prefix, tempdir, "");
@@ -1630,7 +1705,7 @@ char * R_tmpnam(const char * prefix, const char * tempdir)
 
 /* NB for use with multicore: parent and all children share the same
    session directory and run in parallel.
-   So as from 2.14.1, we make sure getpic() is part of the process.
+   So as from 2.14.1, we make sure getpid() is part of the process.
 */
 char * R_tmpnam2(const char *prefix, const char *tempdir, const char *fileext)
 {
@@ -1835,9 +1910,9 @@ SEXP attribute_hidden do_glob(SEXP call, SEXP op, SEXP args, SEXP env)
     {
 	wchar_t *w = globbuf.gl_pathv[i];
 	char *buf;
-	int nb = wcstoutf8(NULL, w, 0);
-	buf = R_AllocStringBuffer(nb+1, &cbuff);
-	wcstoutf8(buf, w, nb+1); buf[nb] = '\0'; /* safety check */
+	int nb = wcstoutf8(NULL, w, INT_MAX);
+	buf = R_AllocStringBuffer(nb, &cbuff);
+	wcstoutf8(buf, w, nb);
 	SET_STRING_ELT(ans, i, mkCharCE(buf, CE_UTF8));
     }
 #else
